/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
* Version: NPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Netscape Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/NPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is mozilla.org code.
*
* The Initial Developer of the Original Code is
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 2002
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*    among others, 
*    David Haas <haasd@cae.wisc.edu>
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the NPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the NPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#import "DHBookmarksController.h"
#import "DHBookmarksManager.h"
#import "DHBookmarksDataSource.h"
#import "DHBookmarkFolder.h"
#import "ImageAndTextCell.h"
#import "HistoryDataSource.h"
#import "NSArray+Utils.h"
#import "ExtendedTableView.h"
#import "DHBookmarksOutlineView.H"


@interface DHBookmarksController(PRIVATE)
- (void) setCanEditSelectedContainerContents:(BOOL)inCanEdit;
@end


@implementation DHBookmarksController

//
// - windowDidLoad
//
// Perform some extra initialization when the window finishes loading
// that we can't do in IB.
//
- (void)windowDidLoad
{
  // the standard table item doesn't handle text and icons. Replace it
  // with a custom cell that does.
  ImageAndTextCell* imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
  [imageAndTextCell setEditable: YES];
  [imageAndTextCell setWraps: NO];
  NSTableColumn* itemNameColumn = [mItemPane tableColumnWithIdentifier: @"title"];
  [itemNameColumn setDataCell:imageAndTextCell];
  NSTableColumn* containerNameColumn = [mContainerPane tableColumnWithIdentifier: @"title"];
  [containerNameColumn setDataCell:imageAndTextCell];

  // set up the font on the item view to be smaller
  NSArray* columns = [mItemPane tableColumns];
  if ( columns ) {
    int numColumns = [columns count];
    NSFont* smallerFont = [NSFont systemFontOfSize:11];
    for ( int i = 0; i < numColumns; ++i )
      [[[columns objectAtIndex:i] dataCell] setFont:smallerFont];    
  }
  // Generic notifications for Bookmark Client
  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
  [nc addObserver:self selector:@selector(bookmarkAdded:) name:DHBookmarkFolderAdditionNotification object:nil];
  [nc addObserver:self selector:@selector(bookmarkRemoved:) name:DHBookmarkFolderDeletionNotification object:nil];
  [nc addObserver:self selector:@selector(bookmarkChanged:) name:DHItemChangedNotification object:nil];
  // register for dragged types
  [mContainerPane registerForDraggedTypes:[NSArray arrayWithObject:@"MozBookmarkType"]];
}

-(void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [super dealloc];
}
//
// - splitViewDidResizeSubviews:
//
// Called when one of the views got resized. We want to ensure that the "add bookmark
// item" button gets lined up with the left edge of the item panel. If the container/item
// split was the one that changed, move it accordingly
//
- (void)splitViewDidResizeSubviews:(NSNotification *)notification
{
  const int kButtonGutter = 8;

  if ( [notification object] == mContainersSplit ) {
    // get the position of the item view relative to the window and set the button
    // to that X value. Yes, this will fall down if the bookmark view is inset from the window
    // but i think we can safely assume it won't be.
    NSRect windowRect = [mItemPane convertRect:[mItemPane bounds] toView:nil];
    NSRect newButtonLocation = [mAddItemButton frame];
    newButtonLocation.origin.x = windowRect.origin.x;
    [mAddItemButton setFrame:newButtonLocation];
    [mAddItemButton setNeedsDisplay:YES];

    // offset by the width of the button and the gutter and we've got the location
    // of the add folder button next to it.
    newButtonLocation.origin.x += newButtonLocation.size.width + kButtonGutter;
    [mAddFolderButton setFrame:newButtonLocation];
    [mAddFolderButton setNeedsDisplay:YES];
  }
}

//
// - splitView:canCollapseSubview:
//
// Called when appkit wants to ask if it can collapse a subview. The only subview
// of our splits that we allow to be hidden is the search panel.
//
- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
  BOOL retVal = NO;
  // subview will be a NSScrollView, so we have to get the superview of the
  // search pane for comparison.
  if ( sender == mItemSearchSplit && subview == [mSearchPane superview] )
    retVal = YES;
  return retVal;
}


- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedCoord ofSubviewAt:(int)offset
{
  const int kMinimumContainerSplitWidth = 150;
  float retVal = proposedCoord;
  if ( sender == mContainersSplit )
    retVal = kMinimumContainerSplitWidth;
  return retVal;
}

- (void) focus
{
  [[mItemPane window] makeFirstResponder:mItemPane];
}

- (IBAction) changeContainer:(id)aSender
{
  [self selectContainer:[aSender clickedRow]];
}

- (void) setCanEditSelectedContainerContents:(BOOL)inCanEdit
{
  [mItemPane setAllowsEditing:inCanEdit];
  // update buttons
  [mAddItemButton setEnabled:inCanEdit];
  [mAddFolderButton setEnabled:inCanEdit];
  [mInfoButton setEnabled:inCanEdit];
}

- (void) selectContainer:(int)inRowIndex
{
  [mContainerPane selectRow:inRowIndex byExtendingSelection:NO];
  if ( inRowIndex == kHistoryIndex ) {
    [mItemPane setDataSource:mHistorySource];
    [mItemPane setDelegate:mHistorySource];
    [mHistorySource ensureDataSourceLoaded];
    [mItemPane reloadData];
    [self setCanEditSelectedContainerContents:NO];
    [mItemPane setTarget:mHistorySource];
    [mItemPane setDoubleAction: @selector(openHistoryItem:)];
    [mItemPane setDeleteAction: @selector(deleteHistoryItems:)];
  } else {
    [mItemPane setDataSource:mBookmarksSource];
    [mItemPane setDelegate:mBookmarksSource];
    DHBookmarkFolder *activeCollection = [[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] objectAtIndex:inRowIndex];
    [mBookmarksSource setActiveCollection:activeCollection];
    [mBookmarksSource ensureBookmarks];
    [mBookmarksSource restoreFolderExpandedStates];
    [mItemPane setTarget:mBookmarksSource];
    [mItemPane setDoubleAction: @selector(openBookmark:)];
    if ([activeCollection isSmartFolder])
      [self setCanEditSelectedContainerContents:NO];
    else
      [self setCanEditSelectedContainerContents:YES];
    [mItemPane setDeleteAction: @selector(deleteBookmarks:)];
  }
}

- (void) selectLastContainer
{
  // we need to call selectContainer: in order to get all the appropriate
  // stuff hooked up.
  [self selectContainer:[mContainerPane selectedRow]];
}

-(IBAction)addCollection:(id)aSender
{
  DHBookmarkFolder *rootFolder = [[DHBookmarksManager sharedBookmarksManager] rootBookmarks];
  DHBookmarkFolder *aFolder = [rootFolder addBookmarkFolder];
  [aFolder setTitle:NSLocalizedString(@"Untitled Folder",@"Untitled Folder")];
  unsigned index = [rootFolder indexOfObjectIdenticalTo:aFolder];
  [self selectContainer:index];
  [mContainerPane editColumn:0 row:index withEvent:nil select:YES];
}


#pragma mark -
//
// table view delegates
//
- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
  if ( tableView == mContainerPane )
    return [[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] count];
  return 0;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
  id itemString = nil;
  if ( tableView == mContainerPane ) {
    return [[[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] objectAtIndex:row] title];
  }
  else if ( tableView == mSearchPane ) {
    if ( [[tableColumn identifier] isEqualToString:@"title"] )
      itemString = @"<Search result here>";
    else
      itemString = @"<URL result here>";
  }
  return itemString;
}

- (void)tableView:(NSTableView *)inTableView willDisplayCell:(id)inCell forTableColumn:(NSTableColumn *)inTableColumn row:(int)inRowIndex
{
  if ( inTableView == mContainerPane ) {
    DHBookmarkFolder *aFolder = [[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] objectAtIndex:inRowIndex];
    [inCell setImage:[aFolder icon]];
  }
}

- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
  if (aTableView == mContainerPane) {
    if (rowIndex >= (int)[[DHBookmarksManager sharedBookmarksManager] firstUserCollection])
      return YES;
    return NO;
  }
  return NO;
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row;
{
  if (tableView == mContainerPane) {
    DHBookmarkFolder *aFolder = [[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] objectAtIndex:row];
    [aFolder setTitle:object];
  }
}

- (BOOL)tableView:(NSTableView *)tv writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pboard
{
  if (tv == mContainerPane) {
    int count = [rows count];
    if (count == 0)
      return NO;
    NSMutableArray *itemArray = [[NSMutableArray alloc] initWithCapacity:count];
    DHBookmarksManager *manager = [DHBookmarksManager sharedBookmarksManager];
    NSEnumerator *enumerator = [rows objectEnumerator];
    unsigned firstUserCollection = [manager firstUserCollection];
    DHBookmarkFolder *rootFolder = [manager rootBookmarks];
    int rowVal;
    id aRow;
    while ((aRow = [enumerator nextObject])) {
      rowVal = [aRow intValue];
      if (rowVal > (int)firstUserCollection)
        [itemArray addObject:[rootFolder objectAtIndex:rowVal]];
    }
    if ([itemArray count] == 0) {
      [itemArray release];
      return NO;
    }
    // Pack pointers to bookmark items into this array
    NSArray *pointerArray = [NSArray dataArrayFromPointerArrayForMozBookmarkDrop:itemArray];
    [itemArray release];
    [pboard declareTypes:[NSArray arrayWithObject:@"MozBookmarkType"] owner:self];
    [pboard setPropertyList:pointerArray forType:@"MozBookmarkType"];
    return YES;
  }
  return NO;
}

- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
  NSArray* types = [[info draggingPasteboard] types];
  if ([types containsObject:@"MozBookmarkType"] && tv == mContainerPane) {
    DHBookmarksManager *manager = [DHBookmarksManager sharedBookmarksManager];
    DHBookmarkFolder *dropFolder = nil;
    if ((row < 2) && (op == NSTableViewDropOn)) 
      dropFolder = [[manager rootBookmarks] objectAtIndex:row];
    else if (row >= (int)[manager firstUserCollection]) {
      if (op == NSTableViewDropAbove)
        dropFolder = [manager rootBookmarks];
      else
        dropFolder = [[manager rootBookmarks] objectAtIndex:row];
    }
    if (dropFolder) {
      NSArray *draggedItems = [NSArray pointerArrayFromDataArrayForMozBookmarkDrop:[[info draggingPasteboard] propertyListForType: @"MozBookmarkType"]];
      BOOL isOK = [manager isDropValid:draggedItems toFolder:dropFolder];
      return (isOK) ? NSDragOperationGeneric: NSDragOperationNone;
    }
  }
  return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
  NSArray *types = [[info draggingPasteboard] types];
  if ([types containsObject:@"MozBookmarkType"] && tv == mContainerPane) {
    DHBookmarkFolder *dropFolder;
    if (op == NSTableViewDropAbove)
      dropFolder = [[DHBookmarksManager sharedBookmarksManager] rootBookmarks];
    else
      dropFolder = [[[DHBookmarksManager sharedBookmarksManager] rootBookmarks] objectAtIndex:row];
    BOOL isCopy = ([info draggingSourceOperationMask] == NSDragOperationCopy);
    NSArray *draggedItems = [NSArray pointerArrayFromDataArrayForMozBookmarkDrop:[[info draggingPasteboard] propertyListForType: @"MozBookmarkType"]];
    NSEnumerator *enumerator = [draggedItems objectEnumerator];
    id aKid;
    while ((aKid = [enumerator nextObject])) {
      if (isCopy) {
        [[aKid parent] copyChild:aKid toBookmarkFolder:dropFolder atIndex:row];
      } else {
        [[aKid parent] moveChild:aKid toBookmarkFolder:dropFolder atIndex:row];
      }
    }
    return YES;
  }
  return NO;
}


#pragma mark -
//
// DHBookmarksClient protocol
//
- (void)bookmarkAdded:(NSNotification *)note
{
  DHBookmarkFolder *rootFolder = [note object];
  if ((rootFolder == [[DHBookmarksManager sharedBookmarksManager] rootBookmarks]))
  {
    [mContainerPane reloadData];
    DHBookmarkFolder *updatedFolder = [[note userInfo] objectForKey:DHBookmarkFolderChildKey];
    [self selectContainer:[rootFolder indexOfObjectIdenticalTo:updatedFolder]];
  }
}
- (void)bookmarkRemoved:(NSNotification *)note
{
  if (([note object] == [[DHBookmarksManager sharedBookmarksManager] rootBookmarks]))
    [mContainerPane reloadData];
}
- (void)bookmarkChanged:(NSNotification *)note
{
  if (([note object] == [[DHBookmarksManager sharedBookmarksManager] rootBookmarks]))
    [mContainerPane reloadData];
}

@end

