/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
* Version: NPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Netscape Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/NPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is mozilla.org code.
*
* The Initial Developer of the Original Code is
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 2002
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*			Simon Fraser <sfraser@netscape.com>
*			Max Horn <max@quendi.de>
*           David Haas <haasd@cae.wisc.edu>
*
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the NPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the NPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#import "NSString+Utils.h"
#import "NSArray+Utils.h"
#import "NSPasteboard+Utils.h"
#import "DHBookmarksManager.h"
#import "DHBookmarksDataSource.h"
#import "DHBookmarkInfoController.h"
#import "DHBookmarkFolder.h"
#import "DHBookmark.h"
#import "ExtendedOutlineView.h"
#import "MainController.h"
#import "BrowserWindowController.h"
#import "PreferenceManager.h"


@implementation DHBookmarksDataSource

-(id) init
{
  if ( (self = [super init]) )
  {
    mCachedHref = nil;
    // Generic notifications for Bookmark Client
    NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
    [nc addObserver:self selector:@selector(bookmarkAdded:) name:DHBookmarkFolderAdditionNotification object:nil];
    [nc addObserver:self selector:@selector(bookmarkRemoved:) name:DHBookmarkFolderDeletionNotification object:nil];
    [nc addObserver:self selector:@selector(bookmarkChanged:) name:DHItemChangedNotification object:nil];
  }
  return self;
}

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [mCachedHref release];
  [mExpandedStatus release];
  [mActiveRootCollection release];
  [super dealloc];
}

-(void) awakeFromNib
{
  // make sure these are disabled at the start since the outliner
  // starts off with no selection.
  [mEditBookmarkButton setEnabled:NO];
  [mDeleteBookmarkButton setEnabled:NO];
}

-(NSMutableDictionary *)expandedStatusDictionary
{
  if (mExpandedStatus)
    return mExpandedStatus;
  mExpandedStatus = [[NSMutableDictionary alloc] initWithCapacity:20];
  return mExpandedStatus;
}

-(void) ensureBookmarks
{
  // from ensure bookmarks
  [mOutlineView setTarget: self];
  [mOutlineView setDoubleAction: @selector(openBookmark:)];
  [mOutlineView setDeleteAction: @selector(deleteBookmarks:)];
  [mOutlineView reloadData];
  [self restoreFolderExpandedStates];
}


-(IBAction)addBookmark:(id)aSender
{
  [self addItem: aSender useSelection: YES isFolder: NO URL:nil title:nil];
}

-(IBAction)addFolder:(id)aSender
{
  [self addItem: aSender useSelection: YES isFolder: YES URL:nil title:nil];
}

-(void)addItem:(id)aSender useSelection:(BOOL)aUseSel isFolder:(BOOL)aIsFolder URL:(NSString*)aURL title:(NSString*)aTitle
{
  // We use the selected item to determine the parent only if aUseSel is YES.
  DHBookmarkFolder *parentFolder = nil;
  DHItem* item = nil;
  if (aUseSel && ([mOutlineView numberOfSelectedRows] == 1))
  {
    // There is only one selected row.  If it is a folder, use it as our parent.
    // Otherwise, use selected row's parent.
    int index = [mOutlineView selectedRow];
    item = [mOutlineView itemAtRow: index];
    if ([item isKindOfClass:[DHBookmarkFolder class]])
      parentFolder = item;
    else
      parentFolder = [item parent];
  } else 
    parentFolder = [self activeCollection];
  [self addItem:aSender withParent:parentFolder isFolder:aIsFolder URL:aURL title:aTitle];
}

-(void)addItem:(id)aSender withParent:(DHBookmarkFolder*)parent isFolder:(BOOL)aIsFolder URL:(NSString*)aURL title:(NSString*)aTitle
{
  NSString* titleString = aTitle;
  NSString* urlString   = aURL;

  if (!aIsFolder)
  {
    // If no URL and title were specified, get them from the current page.
    if (!aURL || !aTitle)
      [[mBrowserWindowController getBrowserWrapper] getTitle:&titleString andHref:&urlString];

    mCachedHref = [NSString stringWithString:urlString];
    [mCachedHref retain];

  } else {   // Folder
    mCachedHref = nil;
    titleString = NSLocalizedString(@"NewBookmarkFolder", @"");
  }

  NSTextField* textField  = [mBrowserWindowController getAddBookmarkTitle];
  NSString* bookmarkTitle = titleString;
  NSString* cleanedTitle  = [bookmarkTitle stringByReplacingCharactersInSet:[NSCharacterSet controlCharacterSet] withString:@" "];

  [textField setStringValue: cleanedTitle];

  [mBrowserWindowController cacheBookmarkVC: self];

  // Show/hide the bookmark all tabs checkbox as appropriate.
  NSTabView* tabView = [mBrowserWindowController getTabBrowser];
  id checkbox = [mBrowserWindowController getAddBookmarkCheckbox];
  BOOL hasSuperview = [checkbox superview] != nil;
  if (aIsFolder && hasSuperview) {
    // Just don't show it at all.
    [checkbox removeFromSuperview];
    [checkbox retain];
  }
  else if (!aIsFolder && !hasSuperview) {
    // Put it back in.
    [[[mBrowserWindowController getAddBookmarkSheetWindow] contentView] addSubview: checkbox];
    [checkbox autorelease];
  }

  // Enable the bookmark all tabs checkbox if appropriate.
  if (!aIsFolder)
    [[mBrowserWindowController getAddBookmarkCheckbox] setEnabled: ([tabView numberOfTabViewItems] > 1)];

  // Build up the folder list.
  NSPopUpButton* popup = [mBrowserWindowController getAddBookmarkFolder];
  [popup removeAllItems];
  DHBookmarkFolder* rootFolder = [[DHBookmarksManager sharedBookmarksManager] rootBookmarks];
  [rootFolder buildFlatFolderList:[popup menu] depth:1];

  int itemIndex = [popup indexOfItemWithRepresentedObject:parent];
  if (itemIndex != -1)
    [popup selectItemAtIndex:itemIndex];

  [popup synchronizeTitleAndSelectedItem];

  [NSApp beginSheet: [mBrowserWindowController getAddBookmarkSheetWindow]
     modalForWindow: [mBrowserWindowController window]
      modalDelegate: nil //self
     didEndSelector: nil //@selector(sheetDidEnd:)
        contextInfo: nil];
}

-(void)endAddBookmark: (int)aCode
{
  if (aCode == 0)
    return;

  BOOL isGroup = NO;
  id checkbox = [mBrowserWindowController getAddBookmarkCheckbox];
  if (([checkbox superview] != nil) && [checkbox isEnabled] && ([checkbox state] == NSOnState))
  {
    [mCachedHref release];
    mCachedHref = nil;
    isGroup = YES;
  }

  NSString* titleString = [[mBrowserWindowController getAddBookmarkTitle] stringValue];
  NSPopUpButton* popup = [mBrowserWindowController getAddBookmarkFolder];
  NSMenuItem* selectedItem = [popup selectedItem];
  DHBookmarkFolder *parentFolder = [selectedItem representedObject];

  if (isGroup)
  {
    DHBookmarkFolder *newGroup = [parentFolder addBookmarkFolder:titleString inPosition:[parentFolder count] isGroup:YES];
    id tabBrowser = [mBrowserWindowController getTabBrowser];
    int count     = [tabBrowser numberOfTabViewItems];
    for (int i = 0; i < count; i++)
    {
      BrowserWrapper* browserWrapper = (BrowserWrapper*)[[tabBrowser tabViewItemAtIndex: i] view];
      NSString* titleString = nil;
      NSString* hrefString = nil;
      [browserWrapper getTitle:&titleString andHref:&hrefString];
      [newGroup addBookmark:titleString url:hrefString inPosition:i isSeparator:NO];
    }
  }
  else
  {
    if (mCachedHref)
    {
      [parentFolder addBookmark:titleString url:mCachedHref inPosition:[parentFolder count] isSeparator:NO];
      [mCachedHref release];
      mCachedHref = nil;
    }
    else
      [parentFolder addBookmarkFolder:titleString inPosition:[parentFolder count] isGroup:NO];
  }
}

-(IBAction)deleteBookmarks: (id)aSender
{
  int index = [mOutlineView selectedRow];
  if (index == -1)
    return;

  // A cheap way of having to avoid scanning the list to remove children is to have the
  // outliner collapse all items that are being deleted. This will cull the selection
  // for us and eliminate any children that happened to be selected.

  BOOL allCollapsed = NO;
  id doomedItem;
  NSEnumerator* selRows;
  while (!allCollapsed) {
    allCollapsed = YES;
    selRows = [mOutlineView selectedRowEnumerator];
    while (allCollapsed && (doomedItem = [selRows nextObject])) {
      doomedItem = [mOutlineView itemAtRow:[doomedItem intValue]];
      if ([mOutlineView isItemExpanded:doomedItem]) {
        allCollapsed = NO;
        [mOutlineView collapseItem:doomedItem];
      }
    }
  }

  // create array of items we need to delete. Deleting items out of of the
  // selection array is problematic for some reason.
  NSMutableArray *itemsToDelete = [[NSMutableArray alloc] init];
  selRows = [mOutlineView selectedRowEnumerator];
  for (NSNumber* currIndex = [selRows nextObject];
       currIndex != nil;
       currIndex = [selRows nextObject]) {
    index = [currIndex intValue];
    DHItem* item = [mOutlineView itemAtRow: index];
    [itemsToDelete addObject: item];
  }

  // delete all bookmarks that are in our array
  int count = [itemsToDelete count];
  for (int i = 0; i < count; i++) {
    doomedItem = [itemsToDelete objectAtIndex:i];
    [[doomedItem parent] deleteChild:doomedItem];
  }
  [itemsToDelete release];

  // restore selection to location near last item deleted or last item
  int total = [mOutlineView numberOfRows];
  if (index >= total)
    index = total - 1;
  [mOutlineView selectRow: index byExtendingSelection: NO];
}

-(IBAction)openBookmark: (id)aSender
{
  int index = [mOutlineView selectedRow];
  if (index == -1)
    return;

  id item = [mOutlineView itemAtRow: index];
  if (!item)
    return;

  if ([item isKindOfClass:[DHBookmark class]]) {
    // if the command key is down, follow the command-click pref
    if (([[NSApp currentEvent] modifierFlags] & NSCommandKeyMask) &&
        [[PreferenceManager sharedInstance] getBooleanPref:"browser.tabs.opentabfor.middleclick" withSuccess:NULL])
    {
      BOOL loadInBackground = [[PreferenceManager sharedInstance] getBooleanPref:"browser.tabs.loadInBackground" withSuccess:NULL];
      [mBrowserWindowController openNewTabWithURL:[item url] referrer:nil loadInBackground: loadInBackground];
    }
    else
      [[mBrowserWindowController getBrowserWrapper] loadURI:[(DHBookmark *)item url] referrer:nil flags:NSLoadFlagsNone activate:YES];
  } else if ([item isKindOfClass:[DHBookmarkFolder class]]) {
    if ([item isGroup])
      [mBrowserWindowController openTabGroup:[item childURLs] replaceExistingTabs:YES];
    else if ([mOutlineView isItemExpanded:item])
      [mOutlineView collapseItem: item];
    else
      [mOutlineView expandItem: item];
  }
}

-(IBAction)openBookmarkInNewTab:(id)aSender
{
  int index = [mOutlineView selectedRow];
  if (index == -1)
    return;

  if ([mOutlineView numberOfSelectedRows] == 1) {
    id item = [mOutlineView itemAtRow: index];
    if ([item isKindOfClass:[DHBookmark class]]) {
      BOOL loadInBackground = [[PreferenceManager sharedInstance] getBooleanPref:"browser.tabs.loadInBackground" withSuccess:NULL];
      [mBrowserWindowController openNewTabWithURL:[item url] referrer:nil loadInBackground: loadInBackground];
    }
  }
}

-(IBAction)openBookmarkInNewWindow:(id)aSender
{
  int index = [mOutlineView selectedRow];
  if (index == -1)
    return;
  
  if ([mOutlineView numberOfSelectedRows] == 1)
  {
    BOOL loadInBackground = [[PreferenceManager sharedInstance] getBooleanPref:"browser.tabs.loadInBackground" withSuccess:NULL];
    id item = [mOutlineView itemAtRow: index];
    if ([item isKindOfClass:[DHBookmark class]])
      [mBrowserWindowController openNewWindowWithURL:[item url] referrer: nil loadInBackground: loadInBackground];
    else if ([item isKindOfClass:[DHBookmarkFolder class]]) {
      if ([item isGroup])
        [mBrowserWindowController openNewWindowWithGroupURLs:[item childURLs] loadInBackground:loadInBackground];
    }
  }
}

-(IBAction)showBookmarkInfo:(id)aSender
{
  DHBookmarkInfoController *bic = [DHBookmarkInfoController sharedBookmarkInfoController];
  int index = [mOutlineView selectedRow];
  DHItem* item = [mOutlineView itemAtRow: index];
  [bic setBookmark:item];
  [bic showWindow:bic];
}

- (BOOL)isExpanded:(id)anItem
{
  NSMutableDictionary *dict = [self expandedStatusDictionary];
  NSNumber *aBool = [dict objectForKey:[NSNumber numberWithUnsignedInt:[anItem hash]]];
  if (aBool) 
    return [aBool boolValue];
  return NO;
}

-(void) setItem:(DHBookmarkFolder *)anItem isExpanded:(BOOL)aBool
{
  NSMutableDictionary *dict = [self expandedStatusDictionary];
  [dict setObject:[NSNumber numberWithBool:aBool] forKey:[NSNumber numberWithUnsignedInt:[anItem hash]]];
}

- (void)restoreFolderExpandedStates
{
  int curRow = 0;
  while (curRow < [mOutlineView numberOfRows])
  {
    id item = [mOutlineView itemAtRow:curRow];
    if ([item isKindOfClass:[DHBookmarkFolder class]])
    {
      if ([self isExpanded:item])
        [mOutlineView expandItem: item];
      else
        [mOutlineView collapseItem: item];
    }
    curRow ++;
  }
}

-(void) setActiveCollection:(DHBookmarkFolder *)aFolder
{
  [aFolder retain];
  [mActiveRootCollection release];
  mActiveRootCollection = aFolder;
}

-(DHBookmarkFolder *)activeCollection
{
  return mActiveRootCollection;
}

#pragma mark -

// BookmarksClient protocol

- (void)bookmarkAdded:(NSNotification *)note
{
  DHBookmarkFolder *aFolder = [note object];
  if (aFolder == mActiveRootCollection)
    aFolder = nil;
  [self reloadDataForItem:aFolder reloadChildren:YES];
}

- (void)bookmarkRemoved:(NSNotification *)note
{
  DHBookmarkFolder *aFolder = [note object];
  if (aFolder == mActiveRootCollection) 
    aFolder = nil;
  [self reloadDataForItem:aFolder reloadChildren:YES];
}

- (void)bookmarkChanged:(NSNotification *)note
{
  [self reloadDataForItem:[note object] reloadChildren:NO];
}

-(BOOL) isExpandedInView:(NSOutlineView *)aView
{
  return NO;
}

#pragma mark -

//
// outlineView:shouldEditTableColumn:item: (delegate method)
//
// Called by the outliner to determine whether or not we should allow the
// user to edit this item. We always return NO, because we invoke the
// edit methods manually.
//
- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
  return NO;	//([[tableColumn identifier] isEqualToString:@"name"]);
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
  if (item) 
    return [item objectAtIndex:index];
  return [[self activeCollection] objectAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
  if ([item isKindOfClass:[DHBookmarkFolder class]] || !(item))
    return YES;
  return NO;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
  if (item) 
    return [item count];
  return [[self activeCollection] count];
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
  id retValue = nil;
  NS_DURING
    retValue = [item valueForKey:[tableColumn identifier]];
  NS_HANDLER
    if ([item isKindOfClass:[DHBookmarkFolder class]] && [[tableColumn identifier] isEqualToString:@"url"]) {
      NSString* itemCountStr = [NSString stringWithFormat:NSLocalizedString(@"Contains Items", @"%u Items"),[item count]];
      NSDictionary* colorAttributes = [NSDictionary dictionaryWithObject:[NSColor disabledControlTextColor] forKey:NSForegroundColorAttributeName];
      retValue = [[[NSAttributedString alloc] initWithString:itemCountStr attributes:colorAttributes] autorelease];
    } else
      retValue = nil;
  NS_ENDHANDLER
    return retValue;
}      

- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
  // set the image on the name column. the url column doesn't have an image.
  if ([[tableColumn identifier] isEqualToString: @"title"])
    [cell setImage:[item icon]];
}

- (void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
  NS_DURING
    [item takeValue:object forKey:[tableColumn identifier]];
  NS_HANDLER
    return;
  NS_ENDHANDLER
}

- (BOOL)outlineView:(NSOutlineView *)ov writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard
{
  int count = [items count];
  if (count == 0)
    return NO;
  // Pack pointers to bookmark items into this array.
  NSArray *pointerArray = [NSArray dataArrayFromPointerArrayForMozBookmarkDrop:items];
  if (count == 1) {
    id aBookmark = [items objectAtIndex:0];
    if ([aBookmark isKindOfClass:[DHBookmark class]]) {
      [pboard declareURLPasteboardWithAdditionalTypes:[NSArray arrayWithObject:@"MozBookmarkType"] owner:self];
      [pboard setDataForURL:[aBookmark url] title:[aBookmark title]];
      [pboard setPropertyList:pointerArray forType:@"MozBookmarkType"];
      return YES;
    }
  }
  // it's either a folder or we've got more than 1 thing. ship it
  // out as MozBookmarkType
  [pboard declareTypes:[NSArray arrayWithObject:@"MozBookmarkType"] owner: self];
  [pboard setPropertyList: pointerArray forType:@"MozBookmarkType"];
  return YES;
}


- (NSDragOperation)outlineView:(NSOutlineView*)ov validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)index
{
  NSArray* types = [[info draggingPasteboard] types];

  //  if the index is -1, deny the drop
  if (index == NSOutlineViewDropOnItemIndex)
    return NSDragOperationNone;

  if ([types containsObject: @"MozBookmarkType"])
  {
    NSArray *draggedItems = [NSArray pointerArrayFromDataArrayForMozBookmarkDrop:[[info draggingPasteboard] propertyListForType: @"MozBookmarkType"]];
    DHBookmarkFolder* parent = (item) ? item : [self activeCollection];
    BOOL isOK = [[DHBookmarksManager sharedBookmarksManager] isDropValid:draggedItems toFolder:parent];
    return (isOK) ? NSDragOperationGeneric : NSDragOperationNone;
  }

  if ([types containsObject: NSStringPboardType] ||
      [types containsObject: NSURLPboardType]    ||
      [types containsObject: @"MozURLType"]      )
    return NSDragOperationGeneric;

  return NSDragOperationNone;
}


- (BOOL)outlineView:(NSOutlineView*)ov acceptDrop:(id <NSDraggingInfo>)info item:(id)item childIndex:(int)index
{
  NSArray*      types  = [[info draggingPasteboard] types];
  BOOL          isCopy = ([info draggingSourceOperationMask] == NSDragOperationCopy);
  if (!item)
    item = [self activeCollection];

  if ([types containsObject: @"MozBookmarkType"])
  {
    NSArray *draggedItems = [NSArray pointerArrayFromDataArrayForMozBookmarkDrop:[[info draggingPasteboard] propertyListForType: @"MozBookmarkType"]];
    NSEnumerator *enumerator = [draggedItems objectEnumerator];
    id aKid;
    while ((aKid = [enumerator nextObject])) {
      if (isCopy) {
        [[aKid parent] copyChild:aKid toBookmarkFolder:item atIndex:index];
      } else {
        [[aKid parent] moveChild:aKid toBookmarkFolder:item atIndex:index];
      }
    }
    return YES;
  }
  else if ([types containsObject: @"MozURLType"])
  {
    NSDictionary* proxy = [[info draggingPasteboard] propertyListForType: @"MozURLType"];
    [item addBookmark:[proxy objectForKey:@"title"] url:[proxy objectForKey:@"url"] inPosition:index isSeparator:NO];
    return YES;
  }
  else if ([types containsObject: NSStringPboardType])
  {
    NSString* draggedText = [[info draggingPasteboard] stringForType:NSStringPboardType];
    [item addBookmark:draggedText url:draggedText inPosition:index isSeparator:NO];
    return YES;
  }
  else if ([types containsObject: NSURLPboardType])
  {
    NSURL*	urlData = [NSURL URLFromPasteboard:[info draggingPasteboard]];
    [item addBookmark:[urlData absoluteString] url:[urlData absoluteString] inPosition:index isSeparator:NO];
    return YES;
  }
  /* may want to move definition of this type into header of NSPasteboard+Utils?
  else if ([types containsObject: kCorePasteboardFlavorType_url])
  {
    NSString *urlString = [[NSString alloc] initWithData:[[info draggingPasteboard] dataForType:kCorePasteboardFlavorType_url] encoding:NSUTF8StringEncoding];
    // check to see if we have a title, too.
    NSString *titleString = nil;
    if ([types containsObject: kCorePasteboardFlavorType_urln])
      titleString = [[NSString alloc] initWithData:[[info draggingPasteboard] dataForType:kCorePasteboardFlavorType_urln] encoding:NSUTF8StringEncoding];
    else
      titleString = [[NSString alloc] init];
    [item addBookmark:titleString url:urlString inPosition:index isSeparator:NO];
    [urlString release];
    [titleString release];
    dropHandled = YES;
  }
  */
  return NO;
}

- (NSString *)outlineView:(NSOutlineView *)outlineView tooltipStringForItem:(id)item
{
  if ([item isKindOfClass:[DHBookmark class]]) {
    if ([[item description] length] > 0)
      return [NSString stringWithFormat:@"%@\n%@",[item url], [item description]];
    else
      return [item url];
  }
  else if ([item isKindOfClass:[DHBookmarkFolder class]])
    return [item description];
  else
    return nil;
}

/*
 - (NSMenu *)outlineView:(NSOutlineView *)outlineView contextMenuForItem:(id)item
 {
   // TODO - return (custom?) context menu for item here.
   // Note that according to HIG, there should never be disabled items in
   // a context menu - instead, items that do not apply should be removed.
   // We could nicely do that here.
 }
 */

- (void)reloadDataForItem:(id)item reloadChildren: (BOOL)aReloadChildren
{
  if (!item)
    [mOutlineView reloadData];
  else
    [mOutlineView reloadItem: item reloadChildren: aReloadChildren];  
}

- (BOOL)haveSelectedRow
{
  return ([mOutlineView selectedRow] != -1);
}

-(void)outlineViewSelectionDidChange: (NSNotification*) aNotification
{
  DHBookmarkInfoController *bic = [DHBookmarkInfoController sharedBookmarkInfoController];
  int index = [mOutlineView selectedRow];
  if (index == -1)
  {
    [mEditBookmarkButton setEnabled:NO];
    [mDeleteBookmarkButton setEnabled:NO];
    [bic close];
  }
  else
  {
    id item = [mOutlineView itemAtRow: index];
    [mEditBookmarkButton setEnabled:YES];
    if ([item isKindOfClass:[DHBookmarkFolder class]])
      [mDeleteBookmarkButton setEnabled:![item isToolbar]];
    else
      [mDeleteBookmarkButton setEnabled:YES];
    if ([[bic window] isVisible])
      [bic setBookmark:item];
  }
}

-(BOOL)validateMenuItem:(NSMenuItem*)aMenuItem
{
  int  index = [mOutlineView selectedRow];
  BOOL haveSelection = (index != -1);
  BOOL multiSelection = ([mOutlineView numberOfSelectedRows] > 1);
  BOOL isBookmark = NO;
  BOOL isToolbar = NO;
  BOOL isGroup = NO;

  id item = nil;

  if (haveSelection)
    item = [mOutlineView itemAtRow: index];
  if ([item isKindOfClass:[DHBookmark class]])
    isBookmark = YES;
  else if ([item isKindOfClass:[DHBookmarkFolder class]]) {
    isGroup = [item isGroup];
    isToolbar = [item isToolbar];
  }

  // Bookmarks and Bookmark Groups can be opened in a new window
  if (([aMenuItem action] == @selector(openBookmarkInNewWindow:)))
    return (isBookmark || isGroup);

  // Only Bookmarks can be opened in new tabs
  if (([aMenuItem action] == @selector(openBookmarkInNewTab:)))
    return isBookmark && [mBrowserWindowController newTabsAllowed];

  if (([aMenuItem action] == @selector(showBookmarkInfo:)))
    return haveSelection;

  if (([aMenuItem action] == @selector(deleteBookmarks:)))
    return (multiSelection || (haveSelection && !isToolbar));
  
  if (([aMenuItem action] == @selector(addFolder:)))
    return YES;

  return YES;
}

- (void)outlineViewItemDidExpand:(NSNotification *)notification
{
  id item = [[notification userInfo] objectForKey:@"NSObject"];
  [self setItem:item isExpanded:YES];
}

- (void)outlineViewItemDidCollapse:(NSNotification *)notification
{
  id item = [[notification userInfo] objectForKey:@"NSObject"];
  [self setItem:item isExpanded:NO];
}

-(void) outlineViewColumnDidMove:(NSNotification *)note
{
  [[note object] autosaveTableColumns];
}

-(void) outlineViewColumnDidResize:(NSNotification *)note
{
  [[note object] autosaveTableColumns];
}

@end

